********************************************************************************
* Program for survey-based inference on dataset from Malawi
* with six waves, twenty lots, and three yes/no outcomes.
*
* The dataset doesn't include the number of yes responses to the three
* outcomes so this program generates fake data for those.
* 
* This program requires two additional user-written programs:
* svyp (which is a wrapper for doing survey-based inference of proportions
*       and for calculating modified-Wilson and modified Clopper-Pearson 
*       confidence intervals)
*
* ciifix (which is called by svyp and is a slightly modified version of
*          a standard Stata command)
*
* Written by Dale Rhoda, Biostat Global Consulting, July 20, 2016
* E-mail:    Dale.Rhoda@biostatglobal.com
* Skype:     dalerhoda
* Phone:	 +1 614 499-2351
*
* Do not hesitate to contact me with questions.
* 
********************************************************************************

set more off

cd "$folder"

import excel "${folder}\Population and sample data LQAS Malawi_combined age groups.xlsx", sheet("Pop & sample data LQAS Malawi") clear cellrange(A2:AK21)

rename A lotname

gen lotid = _n

rename B pop_total
rename C pop_under5

rename D pop1
replace pop1 = round(pop1,1)
label variable pop1 "Pop: 6w-16 months"

drop E
drop F
rename G n6
drop H
drop I
drop J
drop K
drop L
rename M n5
drop N
drop O
drop P
drop Q
rename R n4
drop S
drop T
drop U
drop V
rename W n3
drop X
drop Y
drop Z
drop AA
rename AB n2
drop AC
drop AD
drop AE
drop AF
rename AG n1
drop AH
drop AI
drop AJ
drop AK

* Some of the population figures in the spreadsheet are SMALLER
* than the number of kids sampled...so adjust the population upward
* if necessary
replace pop1 = max(pop1, n1, n2, n3, n4, n5, n6)

* Reshape long so there is one row per lot / wave
reshape long n, i(lotname lotid pop_total pop_under5 pop1) j(wave)

save malawi-step01-premerge_, replace

// Merge vaccination data
use PCV13combined.dta
collapse (sum) pcv1 pcv2 pcv3, by(lqas villageclinicname)
rename villageclinicname lotname
rename lqas wave

merge 1:1 lotname wave using malawi-step01-premerge_.dta 

rename pcv1 yes_count1
rename pcv2 yes_count2
rename pcv3 yes_count3

label variable yes_count1 "Respondents with attribute 1"
label variable yes_count2 "Respondents with attribute 2"
label variable yes_count3 "Respondents with attribute 3"

sort wave lotid

* The code below expects to find a dataset structured like this one
save malawi-step02-postmerge_, replace

* Generate 6 survey datasets
* (one for each wave) and prepare them for 
* survey-based inference using the svyset command

	forvalues j = 1/6 {
		preserve
		keep if wave == `j'
		expand n
		sort lotid
		bysort lotid: gen y1 = _n <= yes_count1
		bysort lotid: gen y2 = _n <= yes_count2
		bysort lotid: gen y3 = _n <= yes_count3
		gen weight = pop1 / n
		svyset _n [pweight = weight], strata(lotid) fpc(pop1)
		save w`j', replace
		restore
	}

* We will hold the results temporarily in a new dataset named 'results'
* which we build, row by row, using the post command
*
* Results will have the estimated proportion of yes responses (phat) 
* for each of the three outcome variables, in addition to lower and 
* upper bounds for 2-sided and 1-sided 95% confidence intervals

capture postclose handle
postfile handle wave lotid 	phat1 lcb_2_sided_95pct_ci1 ucb_2_sided_95pct_ci1 lcb_1_sided_95pct_ci1 ucb_1_sided_95pct_ci1 ///
							phat2 lcb_2_sided_95pct_ci2 ucb_2_sided_95pct_ci2 lcb_1_sided_95pct_ci2 ucb_1_sided_95pct_ci2 ///
							phat3 lcb_2_sided_95pct_ci3 ucb_2_sided_95pct_ci3 lcb_1_sided_95pct_ci3 ucb_1_sided_95pct_ci3 ///
								  using results, replace
								  
								  
	* Loop over waves
	forvalues j = 1/6 {
	
		* Use the appropriate dataset
		use w`j', clear
		
		* Loop over lots
		forvalues k = 1/20 {
			local postit (`j') (`k')

			* Loop over outcome variables
			forvalues l = 1/3 {
				* Use my user-written command, svyp, to estimate the 
				* proportion of yes responses and the 95% CI bounds
				*
				* Use the modified Wilson intervals in case some estimated
				* proportions are quite near 0% or 100%
				svyp y`l' if lotid == `k', method(wilson) level(95)
				local postit `postit' (`=scalar(r(svyp))') ///
				                      (`=scalar(r(lb95))')  (`=scalar(r(ub95))') ///
									  (`=scalar(r(lb90))') (`=scalar(r(ub90))') 
				
			}
			* Post the results for this lot to the results file
			post handle `postit'
		}
		* Estimate & post coverage for all 20 lots combined
		local postit (`j') (0)
		forvalues l = 1/3 {
			svyp y`l' , method(wilson) level(95)
			local postit `postit' (`=scalar(r(svyp))') (`=scalar(r(lb95))') (`=scalar(r(ub95))') (`=scalar(r(lb90))') (`=scalar(r(ub90))') 
		}
		post handle `postit'
	}

capture postclose handle
use results, clear
compress
save, replace

* Merge the results back onto the original dataset
use malawi-step02-postmerge_, clear
merge 1:1 wave lotid using results, nogen
sort wave lotid

replace lotname = "All 20 villages" if lotid == 0

label define noyes 0 "No" 1 "Yes", replace

* If the 1-sided
* lower bound is above your threshold of interest, you can draw a strong
* conclusion that coverage is 95% likely to be higher than the threshold
*
* So I've made a variable to indicate whether coverage is likely to be >= 50%
*
* And likewise if the 1-sided upper confidence bound is below a threshold, 
* you can draw a strong conclusion that coverage is below that threshold;
*
* So I've made a variable to indicate whether coverage is likely to be <= 80%
*
forvalues i = 1/3 {
	gen phat`i'_above_50 = lcb_1_sided_95pct_ci`i' >= 0.5
	order phat`i'_above_50, after(lcb_1_sided_95pct_ci`i')
	label values phat`i'_above_50 noyes
	label variable phat`i'_above_50 "95% confident that phat`i' is >= 50%"
	
	gen phat`i'_below_80 = ucb_1_sided_95pct_ci`i' <= 0.8
	order phat`i'_below_80, after(ucb_1_sided_95pct_ci`i')
	label values phat`i'_below_80 noyes
	label variable phat`i'_below_80 "95% confident that phat`i' if <= 80%"
}

save malawi-step03-results_, replace

* Export to excel
*
* You'll want to go into this spreadsheet and format the results as %.
* (All the columns named 'phat...' and 'lcb...' and 'ucb...')

export excel using malawi-results_combined_age_groups.xlsx, firstrow(var) replace

* Clean up:
* Erase the temporary results dataset
capture erase results.dta
