********************************************************************************
* Programs for survey-based inference on dataset from Malawi
* with six waves, twenty lots, and three yes/no outcomes.
* 
* These programs requires several additional user-written programs
* that are found in a sub-folder named programs_to_make_inchworm_plots.
*
*
* Written by Dale Rhoda, Biostat Global Consulting, July 20, 2016
* E-mail:    Dale.Rhoda@biostatglobal.com
* Skype:     dalerhoda
* Phone:	 +1 614 499-2351
*
* Do not hesitate to contact me with questions.
* 
********************************************************************************

set more off

cd "$folder"

use malawi_svyset_dataset, clear

* We will hold the results temporarily in a new dataset named 'results'
* which we build, row by row, using the post command
*
* Results will have the estimated proportion of yes responses (phat) 
* for each of the three outcome variables, in addition to lower and 
* upper bounds for 2-sided and 1-sided 95% confidence intervals

capture postclose handle
postfile handle cohort wave lotid phat1 n1 neff1 lcb_2_sided_95pct_ci1 ucb_2_sided_95pct_ci1 lcb_1_sided_95pct_ci1 ucb_1_sided_95pct_ci1 ///
								  phat2 n2 neff2 lcb_2_sided_95pct_ci2 ucb_2_sided_95pct_ci2 lcb_1_sided_95pct_ci2 ucb_1_sided_95pct_ci2 ///
								  phat3 n3 neff3 lcb_2_sided_95pct_ci3 ucb_2_sided_95pct_ci3 lcb_1_sided_95pct_ci3 ucb_1_sided_95pct_ci3 ///
								  using results, replace

local cohort_1 cohort == 1
local cohort_2 cohort == 2
local cohort_3 1==1					  
								  
* Loop over cohorts
forvalues i = 1/3 {

	* Loop over waves
	forvalues j = 1/6 {

		* Loop over lots
		forvalues k = 1/20 {
		
			local postit (`i') (`j') (`k')

			* Loop over outcome variables
			forvalues l = 1/3 {
				
				* Use my user-written command, svyp, to estimate the 
				* proportion of yes responses and the 95% CI bounds
				*
				* Use the modified Wilson intervals in case some estimated
				* proportions are quite near 0% or 100%
				
				qui count if !missing(pcv`l') & lotid == `k' & `cohort_`i'' & wave == `j'
				
				if r(N) > 0 {
				
					svyp pcv`l' if lotid == `k' & `cohort_`i'' & wave == `j', method(wilson) level(95)
					local deff = max(1,r(deff))
					local deff = r(deff)
					local postit `postit' (`=100*scalar(r(svyp))') ///
										  (`=scalar(r(N))') ///
										  (`=scalar(r(N))/`deff'') ///
										  (`=100*scalar(r(lb95))') ///
										  (`=100*scalar(r(ub95))') ///
										  (`=100*scalar(r(lb90))') ///
										  (`=100*scalar(r(ub90))') 
				}
				else local postit `postit' (.) (.) (.) (.) (.) (.) (.)
				
			}
			* Post the results for this lot to the results file
			post handle `postit'
		}
		* Estimate & post coverage for all 20 lots combined
		local postit (`i') (`j') (0)
		forvalues l = 1/3 {
			svyp pcv`l' if `cohort_`i'' & wave == `j', method(wilson) level(95)
			local deff = max(1,r(deff))
			local deff = r(deff)
			local postit `postit' (`=100*scalar(r(svyp))') ///
								  (`=scalar(r(N))') ///
			                      (`=scalar(r(N))/`deff'') ///
								  (`=100*scalar(r(lb95))') ///
								  (`=100*scalar(r(ub95))') ///
								  (`=100*scalar(r(lb90))') ///
								  (`=100*scalar(r(ub90))') 
		}
		post handle `postit'
	}
}

capture postclose handle
use results, clear
compress
merge m:1 lotid using facilityname, nogen
replace facilityname = "All 20 Villages" if lotid == 0
save, replace

label define noyes 0 "No" 1 "Yes", replace

* If the 1-sided
* lower bound is above your threshold of interest, you can draw a strong
* conclusion that coverage is 95% likely to be higher than the threshold
*
* So I've made a variable to indicate whether coverage is likely to be >= 50%
*
* And likewise if the 1-sided upper confidence bound is below a threshold, 
* you can draw a strong conclusion that coverage is below that threshold;
*
* So I've made a variable to indicate whether coverage is likely to be <= 80%
*
forvalues i = 1/3 {
	gen phat`i'_above_50 = lcb_1_sided_95pct_ci`i' >= 0.5
	order phat`i'_above_50, after(lcb_1_sided_95pct_ci`i')
	label values phat`i'_above_50 noyes
	label variable phat`i'_above_50 "95% confident that phat`i' is >= 50%"
	
	gen phat`i'_below_80 = ucb_1_sided_95pct_ci`i' <= 0.8
	order phat`i'_below_80, after(ucb_1_sided_95pct_ci`i')
	label values phat`i'_below_80 noyes
	label variable phat`i'_below_80 "95% confident that phat`i' if <= 80%"
}

save malawi-step03-results, replace

* Export to excel
*
* You'll want to go into this spreadsheet and format the results as %.
* (All the columns named 'phat...' and 'lcb...' and 'ucb...')

export excel using malawi-results.xlsx, firstrow(var) replace

* Clean up:

capture erase results.dta

********************************************************************************
* Make inchworm plots with 18 rows each

use malawi-step03-results, clear

replace n1 = 0 if missing(n1)
replace n2 = 0 if missing(n2)
replace n3 = 0 if missing(n3)

local c1 Cohort 1
local c2 Cohort 2
local c3 Both

forvalues i = 0/20 {

	preserve
	
	keep if lotid == `i'
	
	sort cohort wave
	
	capture postclose handle
	
	postfile handle str7 source str25 rowname rownumber str6 param3 str500 param4 str500 param5 str500 param6 str500 param7 str10 outlinecolor str10 outlinepattern str10 areacolor  areaintensity clip str50 rightsidetext using temp, replace
	
	local r 0
	forvalues j = 1/`=_N' {
	
		local ++r
		
		local rstring
		if n3[`j']>0 local rstring `rstring' `=round(`=phat3[`j']',1)'% (N=`=n3[`j']')
		else local rstring `rstring' * (N=0)
		if n2[`j']>0 local rstring `rstring' `=round(`=phat2[`j']',1)'% (N=`=n2[`j']')
		else local rstring `rstring' * (N=0)
		if n1[`j']>0 local rstring `rstring' `=round(`=phat1[`j']',1)'% (N=`=n1[`j']')
		else local rstring `rstring' * (N=0)
		
		local ifclause if wave == `=wave[`j']' 
		if `i' > 0 local ifclause `ifclause' & lotid == `=lotid[`j']'
		if inlist(cohort[`j'] ,1,2) local ifclause `ifclause' & cohort == `=cohort[`j']'
	
		if n1[`j'] >0 post handle ("DATASET") ("`c`=cohort[`j']'' : W`=wave[`j']'")    (`r') ("Wilson") ("${folder}/malawi_svyset_dataset") ("pcv1") ("svyset _n [pweight = weight], strata(stratum) fpc(pop) single(certainty)") ("`ifclause'") ("black") ("solid") ("edkblue") (.) (95) ("`rstring'")
		if n2[`j'] >0 post handle ("DATASET") ("")                                     (`r') ("Wilson") ("${folder}/malawi_svyset_dataset") ("pcv2") ("svyset _n [pweight = weight], strata(stratum) fpc(pop) single(certainty)") ("`ifclause'") ("black") ("solid") ("emidblue")  (.) (95) ("")
		if n3[`j'] >0 post handle ("DATASET") ("")                                     (`r') ("Wilson") ("${folder}/malawi_svyset_dataset") ("pcv3") ("svyset _n [pweight = weight], strata(stratum) fpc(pop) single(certainty)") ("`ifclause'") ("black") ("solid") ("eltblue")  (.) (95) ("")
		
	}
	
	local fn `=facilityname[1]'
	
	capture postclose handle
	use temp, clear
	compress
	
	putexcel set "./IW_Facility_Template.xlsx", sheet(command_lines) modify
	putexcel D2 = "`fn',span"
	putexcel clear
	
	export excel IW_Facility_Template.xlsx, sheet("distribution_info") firstrow(variables) sheetreplace
	
	excel_wrapper_for_iwplot_svyp IW_Facility_Template.xlsx
	
	copy IW_Facility_Template.xlsx "IW_`fn'.xlsx", replace
	
	if `i' < 10 graph export "P18_0`i'_`fn'.png", width(2000) replace
	else        graph export  "P18_`i'_`fn'.png", width(2000) replace
	
	restore
	
}

